@@ -21,6 +21,7 @@ from .statement import (
     Sleep,
     SleepUnit,
     Skip,
+    Unzip,
     SortStyle,
     Unskip,
 )
@@ -332,9 +333,9 @@ class QueryResult:
                 hash_compare_error = values[0] != hash_value
 
             if hash_compare_error:
-                expected_result = self.result_label_map.get(query_label)
-                logger.wrong_result_hash(expected_result, self)
-                self.fail_query(query)
+                expected_result = runner.result_label_map.get(query_label)
+                # logger.wrong_result_hash(expected_result, self)
+                context.fail(query)
 
             assert not hash_compare_error
 
@@ -458,6 +459,19 @@ def matches_regex(input: str, actual_str: str) -> bool:
     return regex_matches == should_match
 
 
+def has_external_access(conn):
+    # this is required for the python tester to work, as we make use of replacement scans
+    try:
+        res = conn.sql("select current_setting('enable_external_access')").fetchone()[0]
+        return res
+    except duckdb.TransactionException:
+        return True
+    except duckdb.BinderException:
+        return True
+    except duckdb.InvalidInputException:
+        return True
+
+
 def compare_values(result: QueryResult, actual_str, expected_str, current_column):
     error = False
 
@@ -707,7 +721,7 @@ class SQLLogicContext:
         # Apply a replacement for every registered keyword
         if '__BUILD_DIRECTORY__' in input:
             self.skiptest("Test contains __BUILD_DIRECTORY__ which isnt supported")
-        for key, value in self.keywords.items():
+        for key, value in self.keywords.items().__reversed__():
             input = input.replace(key, value)
         return input
 
@@ -751,6 +765,7 @@ class SQLLogicContext:
             Restart: self.execute_restart,
             HashThreshold: self.execute_hash_threshold,
             Set: self.execute_set,
+            Unzip: self.execute_unzip,
             Loop: self.execute_loop,
             Foreach: self.execute_foreach,
             Endloop: None,  # <-- should never be encountered outside of Loop/Foreach
@@ -815,6 +830,8 @@ class SQLLogicContext:
     def execute_query(self, query: Query):
         assert isinstance(query, Query)
         conn = self.get_connection(query.connection_name)
+        if not has_external_access(conn):
+            self.skiptest("enable_external_access is explicitly disabled by the test")
         sql_query = '\n'.join(query.lines)
         sql_query = self.replace_keywords(sql_query)
 
@@ -885,6 +902,18 @@ class SQLLogicContext:
     def execute_skip(self, statement: Skip):
         self.runner.skip()
 
+    def execute_unzip(self, statement: Unzip):
+        import gzip
+        import shutil
+
+        source = self.replace_keywords(statement.source)
+        destination = self.replace_keywords(statement.destination)
+
+        with gzip.open(source, 'rb') as f_in:
+            with open(destination, 'wb') as f_out:
+                shutil.copyfileobj(f_in, f_out)
+        print(f"Extracted to '{destination}'")
+
     def execute_unskip(self, statement: Unskip):
         self.runner.unskip()
 
@@ -978,6 +1007,9 @@ class SQLLogicContext:
     def execute_statement(self, statement: Statement):
         assert isinstance(statement, Statement)
         conn = self.get_connection(statement.connection_name)
+        if not has_external_access(conn):
+            self.skiptest("enable_external_access is explicitly disabled by the test")
+
         sql_query = '\n'.join(statement.lines)
         sql_query = self.replace_keywords(sql_query)
 
