@@ -141,97 +141,12 @@ struct TimeConvert {
 };
 
 struct StringConvert {
-	template <class T>
-	static void ConvertUnicodeValueTemplated(T *result, int32_t *codepoints, idx_t codepoint_count, const char *data,
-	                                         idx_t ascii_count) {
-		// we first fill in the batch of ascii characters directly
-		for (idx_t i = 0; i < ascii_count; i++) {
-			result[i] = data[i];
-		}
-		// then we fill in the remaining codepoints from our codepoint array
-		for (idx_t i = 0; i < codepoint_count; i++) {
-			result[ascii_count + i] = codepoints[i];
-		}
-	}
-
-	static PyObject *ConvertUnicodeValue(const char *data, idx_t len, idx_t start_pos) {
-		// slow path: check the code points
-		// we know that all characters before "start_pos" were ascii characters, so we don't need to check those
-
-		// allocate an array of code points so we only have to convert the codepoints once
-		// short-string optimization
-		// we know that the max amount of codepoints is the length of the string
-		// for short strings (less than 64 bytes) we simply statically allocate an array of 256 bytes (64x int32)
-		// this avoids memory allocation for small strings (common case)
-		idx_t remaining = len - start_pos;
-		unique_ptr<int32_t[]> allocated_codepoints;
-		int32_t static_codepoints[64];
-		int32_t *codepoints;
-		if (remaining > 64) {
-			allocated_codepoints = unique_ptr<int32_t[]>(new int32_t[remaining]);
-			codepoints = allocated_codepoints.get();
-		} else {
-			codepoints = static_codepoints;
-		}
-		// now we iterate over the remainder of the string to convert the UTF8 string into a sequence of codepoints
-		// and to find the maximum codepoint
-		int32_t max_codepoint = 127;
-		int sz;
-		idx_t pos = start_pos;
-		idx_t codepoint_count = 0;
-		while (pos < len) {
-			codepoints[codepoint_count] = Utf8Proc::UTF8ToCodepoint(data + pos, sz);
-			pos += sz;
-			if (codepoints[codepoint_count] > max_codepoint) {
-				max_codepoint = codepoints[codepoint_count];
-			}
-			codepoint_count++;
-		}
-		// based on the max codepoint, we construct the result string
-		auto result = PyUnicode_New(start_pos + codepoint_count, max_codepoint);
-		// based on the resulting unicode kind, we fill in the code points
-		auto result_handle = py::handle(result);
-		auto kind = PyUtil::PyUnicodeKind(result_handle);
-		switch (kind) {
-		case PyUnicode_1BYTE_KIND:
-			ConvertUnicodeValueTemplated<Py_UCS1>(PyUtil::PyUnicode1ByteData(result_handle), codepoints,
-			                                      codepoint_count, data, start_pos);
-			break;
-		case PyUnicode_2BYTE_KIND:
-			ConvertUnicodeValueTemplated<Py_UCS2>(PyUtil::PyUnicode2ByteData(result_handle), codepoints,
-			                                      codepoint_count, data, start_pos);
-			break;
-		case PyUnicode_4BYTE_KIND:
-			ConvertUnicodeValueTemplated<Py_UCS4>(PyUtil::PyUnicode4ByteData(result_handle), codepoints,
-			                                      codepoint_count, data, start_pos);
-			break;
-		default:
-			throw NotImplementedException("Unsupported typekind constant '%d' for Python Unicode Compact decode", kind);
-		}
-		return result;
-	}
-
 	template <class DUCKDB_T, class NUMPY_T>
 	static PyObject *ConvertValue(string_t val, NumpyAppendData &append_data) {
 		(void)append_data;
-		// we could use PyUnicode_FromStringAndSize here, but it does a lot of verification that we don't need
-		// because of that it is a lot slower than it needs to be
 		auto data = const_data_ptr_cast(val.GetData());
 		auto len = val.GetSize();
-		// check if there are any non-ascii characters in there
-		for (idx_t i = 0; i < len; i++) {
-			if (data[i] > 127) {
-				// there are! fallback to slower case
-				return ConvertUnicodeValue(const_char_ptr_cast(data), len, i);
-			}
-		}
-		// no unicode: fast path
-		// directly construct the string and memcpy it
-		auto result = PyUnicode_New(len, 127);
-		auto result_handle = py::handle(result);
-		auto target_data = PyUtil::PyUnicodeDataMutable(result_handle);
-		memcpy(target_data, data, len);
-		return result;
+		return PyUnicode_FromStringAndSize(const_char_ptr_cast(data), len);
 	}
 	template <class NUMPY_T, bool PANDAS>
 	static NUMPY_T NullValue(bool &set_mask) {
@@ -373,23 +288,7 @@ struct MapConvert {
 	static py::dict ConvertValue(Vector &input, idx_t chunk_offset, NumpyAppendData &append_data) {
 		auto &client_properties = append_data.client_properties;
 		auto val = input.GetValue(chunk_offset);
-		auto &list_children = ListValue::GetChildren(val);
-
-		auto &key_type = MapType::KeyType(input.GetType());
-		auto &val_type = MapType::ValueType(input.GetType());
-
-		py::list keys;
-		py::list values;
-		for (auto &list_elem : list_children) {
-			auto &struct_children = StructValue::GetChildren(list_elem);
-			keys.append(PythonObject::FromValue(struct_children[0], key_type, client_properties));
-			values.append(PythonObject::FromValue(struct_children[1], val_type, client_properties));
-		}
-
-		py::dict py_struct;
-		py_struct["key"] = keys;
-		py_struct["value"] = values;
-		return py_struct;
+		return PythonObject::FromValue(val, input.GetType(), client_properties);
 	}
 };
 
@@ -583,10 +482,12 @@ static bool ConvertDecimalInternal(NumpyAppendData &append_data, double division
 	auto src_ptr = UnifiedVectorFormat::GetData<DUCKDB_T>(idata);
 	auto out_ptr = reinterpret_cast<double *>(target_data);
 	if (!idata.validity.AllValid()) {
+		bool requires_mask = false;
 		for (idx_t i = 0; i < count; i++) {
 			idx_t src_idx = idata.sel->get_index(i + source_offset);
 			idx_t offset = target_offset + i;
 			if (!idata.validity.RowIsValidUnsafe(src_idx)) {
+				requires_mask = true;
 				target_mask[offset] = true;
 			} else {
 				out_ptr[offset] =
@@ -595,7 +496,7 @@ static bool ConvertDecimalInternal(NumpyAppendData &append_data, double division
 				target_mask[offset] = false;
 			}
 		}
-		return true;
+		return requires_mask;
 	} else {
 		for (idx_t i = 0; i < count; i++) {
 			idx_t src_idx = idata.sel->get_index(i + source_offset);
@@ -681,7 +582,8 @@ void ArrayWrapper::Append(idx_t current_offset, Vector &input, idx_t source_size
 		} else {
 			throw InternalException("Size not supported on ENUM types");
 		}
-	} break;
+		break;
+	}
 	case LogicalTypeId::BOOLEAN:
 		may_have_null = ConvertColumnRegular<bool>(append_data);
 		break;
