@@ -59,6 +59,7 @@
 #include "duckdb/main/relation/query_relation.hpp"
 #include "duckdb/main/extension_util.hpp"
 #include "duckdb/parser/statement/load_statement.hpp"
+#include "duckdb_python/expression/pyexpression.hpp"
 
 #include <random>
 
@@ -66,7 +67,7 @@
 
 namespace duckdb {
 
-shared_ptr<DuckDBPyConnection> DuckDBPyConnection::default_connection = nullptr;       // NOLINT: allow global
+DefaultConnectionHolder DuckDBPyConnection::default_connection;                        // NOLINT: allow global
 DBInstanceCache instance_cache;                                                        // NOLINT: allow global
 shared_ptr<PythonImportCache> DuckDBPyConnection::import_cache = nullptr;              // NOLINT: allow global
 PythonEnvironmentType DuckDBPyConnection::environment = PythonEnvironmentType::NORMAL; // NOLINT: allow global
@@ -210,7 +211,7 @@ static void InitializeConnectionMethods(py::class_<DuckDBPyConnection, shared_pt
 	m.def("unregister", &DuckDBPyConnection::UnregisterPythonObject, "Unregister the view name", py::arg("view_name"));
 	m.def("table", &DuckDBPyConnection::Table, "Create a relation object for the named table", py::arg("table_name"));
 	m.def("view", &DuckDBPyConnection::View, "Create a relation object for the named view", py::arg("view_name"));
-	m.def("values", &DuckDBPyConnection::Values, "Create a relation object from the passed values", py::arg("values"));
+	m.def("values", &DuckDBPyConnection::Values, "Create a relation object from the passed values");
 	m.def("table_function", &DuckDBPyConnection::TableFunction,
 	      "Create a relation object from the named table function with given parameters", py::arg("name"),
 	      py::arg("parameters") = py::none());
@@ -265,15 +266,6 @@ static void InitializeConnectionMethods(py::class_<DuckDBPyConnection, shared_pt
 	      py::arg("binary_as_string") = false, py::kw_only(), py::arg("file_row_number") = false,
 	      py::arg("filename") = false, py::arg("hive_partitioning") = false, py::arg("union_by_name") = false,
 	      py::arg("compression") = py::none());
-	m.def("from_substrait", &DuckDBPyConnection::FromSubstrait, "Create a query object from protobuf plan",
-	      py::arg("proto"));
-	m.def("get_substrait", &DuckDBPyConnection::GetSubstrait, "Serialize a query to protobuf", py::arg("query"),
-	      py::kw_only(), py::arg("enable_optimizer") = true);
-	m.def("get_substrait_json", &DuckDBPyConnection::GetSubstraitJSON,
-	      "Serialize a query to protobuf on the JSON format", py::arg("query"), py::kw_only(),
-	      py::arg("enable_optimizer") = true);
-	m.def("from_substrait_json", &DuckDBPyConnection::FromSubstraitJSON,
-	      "Create a query object from a JSON protobuf plan", py::arg("json"));
 	m.def("get_table_names", &DuckDBPyConnection::GetTableNames, "Extract the required table names from a query",
 	      py::arg("query"));
 	m.def("install_extension", &DuckDBPyConnection::InstallExtension,
@@ -511,17 +503,17 @@ py::list TransformNamedParameters(const case_insensitive_map_t<idx_t> &named_par
 	return new_params;
 }
 
-case_insensitive_map_t<BoundParameterData> TransformPreparedParameters(PreparedStatement &prep,
-                                                                       const py::object &params) {
+case_insensitive_map_t<BoundParameterData> TransformPreparedParameters(const py::object &params,
+                                                                       optional_ptr<PreparedStatement> prep = {}) {
 	case_insensitive_map_t<BoundParameterData> named_values;
 	if (py::is_list_like(params)) {
-		if (prep.named_param_map.size() != py::len(params)) {
+		if (prep && prep->named_param_map.size() != py::len(params)) {
 			if (py::len(params) == 0) {
 				throw InvalidInputException("Expected %d parameters, but none were supplied",
-				                            prep.named_param_map.size());
+				                            prep->named_param_map.size());
 			}
-			throw InvalidInputException("Prepared statement needs %d parameters, %d given", prep.named_param_map.size(),
-			                            py::len(params));
+			throw InvalidInputException("Prepared statement needs %d parameters, %d given",
+			                            prep->named_param_map.size(), py::len(params));
 		}
 		auto unnamed_values = DuckDBPyConnection::TransformPythonParamList(params);
 		for (idx_t i = 0; i < unnamed_values.size(); i++) {
@@ -560,7 +552,7 @@ unique_ptr<QueryResult> DuckDBPyConnection::ExecuteInternal(PreparedStatement &p
 	}
 
 	// Execute the prepared statement with the prepared parameters
-	auto named_values = TransformPreparedParameters(prep, params);
+	auto named_values = TransformPreparedParameters(params, prep);
 	unique_ptr<QueryResult> res;
 	{
 		D_ASSERT(py::gil_check());
@@ -580,6 +572,35 @@ unique_ptr<QueryResult> DuckDBPyConnection::ExecuteInternal(PreparedStatement &p
 	return res;
 }
 
+unique_ptr<QueryResult> DuckDBPyConnection::PrepareAndExecuteInternal(unique_ptr<SQLStatement> statement,
+                                                                      py::object params) {
+	if (params.is_none()) {
+		params = py::list();
+	}
+
+	auto named_values = TransformPreparedParameters(params);
+
+	unique_ptr<QueryResult> res;
+	{
+		D_ASSERT(py::gil_check());
+		py::gil_scoped_release release;
+		unique_lock<std::mutex> lock(py_connection_lock);
+
+		auto pending_query = con.GetConnection().PendingQuery(std::move(statement), named_values, true);
+
+		if (pending_query->HasError()) {
+			pending_query->ThrowError();
+		}
+
+		res = CompletePendingQuery(*pending_query);
+
+		if (res->HasError()) {
+			res->ThrowError();
+		}
+	}
+	return res;
+}
+
 vector<unique_ptr<SQLStatement>> DuckDBPyConnection::GetStatements(const py::object &query) {
 	vector<unique_ptr<SQLStatement>> result;
 	auto &connection = con.GetConnection();
@@ -616,8 +637,7 @@ shared_ptr<DuckDBPyConnection> DuckDBPyConnection::Execute(const py::object &que
 	// FIXME: SQLites implementation says to not accept an 'execute' call with multiple statements
 	ExecuteImmediately(std::move(statements));
 
-	auto prep = PrepareQuery(std::move(last_statement));
-	auto res = ExecuteInternal(*prep, std::move(params));
+	auto res = PrepareAndExecuteInternal(std::move(last_statement), std::move(params));
 
 	// Set the internal 'result' object
 	if (res) {
@@ -901,7 +921,9 @@ PathLike DuckDBPyConnection::GetPathLike(const py::object &object) {
 static void AcceptableCSVOptions(const string &unkown_parameter) {
 	// List of strings to match against
 	const unordered_set<string> valid_parameters = {"header",
+	                                                "strict_mode",
 	                                                "compression",
+	                                                "comment"
 	                                                "sep",
 	                                                "delimiter",
 	                                                "dtype",
@@ -947,9 +969,31 @@ static void AcceptableCSVOptions(const string &unkown_parameter) {
 	}
 	throw InvalidInputException(error.str());
 }
+void ConvertBooleanValue(const py::object &value, string param_name, named_parameter_map_t &bind_parameters) {
+	if (!py::none().is(value)) {
+
+		bool value_as_int = py::isinstance<py::int_>(value);
+		bool value_as_bool = py::isinstance<py::bool_>(value);
+
+		bool converted_value;
+		if (value_as_bool) {
+			converted_value = py::bool_(value);
+		} else if (value_as_int) {
+			if (static_cast<int>(py::int_(value)) != 0) {
+				throw InvalidInputException("read_csv only accepts 0 if '%s' is given as an integer", param_name);
+			}
+			converted_value = true;
+		} else {
+			throw InvalidInputException("read_csv only accepts '%s' as an integer, or a boolean", param_name);
+		}
+		bind_parameters[param_name] = Value::BOOLEAN(converted_value);
+	}
+}
 
 unique_ptr<DuckDBPyRelation> DuckDBPyConnection::ReadCSV(const py::object &name_p, py::kwargs &kwargs) {
 	py::object header = py::none();
+	py::object strict_mode = py::none();
+	py::object auto_detect = py::none();
 	py::object compression = py::none();
 	py::object sep = py::none();
 	py::object delimiter = py::none();
@@ -985,9 +1029,10 @@ unique_ptr<DuckDBPyRelation> DuckDBPyConnection::ReadCSV(const py::object &name_
 	py::object union_by_name = py::none();
 	py::object hive_types = py::none();
 	py::object hive_types_autocast = py::none();
+	py::object comment = py::none();
+
 	for (auto &arg : kwargs) {
 		const auto &arg_name = py::str(arg.first).cast<std::string>();
-
 		if (arg_name == "header") {
 			header = kwargs[arg_name.c_str()];
 		} else if (arg_name == "compression") {
@@ -996,6 +1041,8 @@ unique_ptr<DuckDBPyRelation> DuckDBPyConnection::ReadCSV(const py::object &name_
 			sep = kwargs[arg_name.c_str()];
 		} else if (arg_name == "delimiter") {
 			delimiter = kwargs[arg_name.c_str()];
+		} else if (arg_name == "comment") {
+			comment = kwargs[arg_name.c_str()];
 		} else if (arg_name == "dtype") {
 			dtype = kwargs[arg_name.c_str()];
 		} else if (arg_name == "na_values") {
@@ -1016,6 +1063,8 @@ unique_ptr<DuckDBPyRelation> DuckDBPyConnection::ReadCSV(const py::object &name_
 			timestamp_format = kwargs[arg_name.c_str()];
 		} else if (arg_name == "sample_size") {
 			sample_size = kwargs[arg_name.c_str()];
+		} else if (arg_name == "auto_detect") {
+			auto_detect = kwargs[arg_name.c_str()];
 		} else if (arg_name == "all_varchar") {
 			all_varchar = kwargs[arg_name.c_str()];
 		} else if (arg_name == "normalize_names") {
@@ -1060,6 +1109,8 @@ unique_ptr<DuckDBPyRelation> DuckDBPyConnection::ReadCSV(const py::object &name_
 			hive_types = kwargs[arg_name.c_str()];
 		} else if (arg_name == "hive_types_autocast") {
 			hive_types_autocast = kwargs[arg_name.c_str()];
+		} else if (arg_name == "strict_mode") {
+			strict_mode = kwargs[arg_name.c_str()];
 		} else {
 			AcceptableCSVOptions(arg_name);
 		}
@@ -1077,24 +1128,8 @@ unique_ptr<DuckDBPyRelation> DuckDBPyConnection::ReadCSV(const py::object &name_
 
 	// First check if the header is explicitly set
 	// when false this affects the returned types, so it needs to be known at initialization of the relation
-	if (!py::none().is(header)) {
-
-		bool header_as_int = py::isinstance<py::int_>(header);
-		bool header_as_bool = py::isinstance<py::bool_>(header);
-
-		bool header_value;
-		if (header_as_bool) {
-			header_value = py::bool_(header);
-		} else if (header_as_int) {
-			if ((int)py::int_(header) != 0) {
-				throw InvalidInputException("read_csv only accepts 0 if 'header' is given as an integer");
-			}
-			header_value = true;
-		} else {
-			throw InvalidInputException("read_csv only accepts 'header' as an integer, or a boolean");
-		}
-		bind_parameters["header"] = Value::BOOLEAN(header_value);
-	}
+	ConvertBooleanValue(header, "header", bind_parameters);
+	ConvertBooleanValue(strict_mode, "strict_mode", bind_parameters);
 
 	if (!py::none().is(compression)) {
 		if (!py::isinstance<py::str>(compression)) {
@@ -1197,6 +1232,13 @@ unique_ptr<DuckDBPyRelation> DuckDBPyConnection::ReadCSV(const py::object &name_
 		bind_parameters["quote"] = Value(py::str(quotechar));
 	}
 
+	if (!py::none().is(comment)) {
+		if (!py::isinstance<py::str>(comment)) {
+			throw InvalidInputException("read_csv only accepts 'comment' as a string");
+		}
+		bind_parameters["comment"] = Value(py::str(comment));
+	}
+
 	if (!py::none().is(escapechar)) {
 		if (!py::isinstance<py::str>(escapechar)) {
 			throw InvalidInputException("read_csv only accepts 'escapechar' as a string");
@@ -1221,6 +1263,23 @@ unique_ptr<DuckDBPyRelation> DuckDBPyConnection::ReadCSV(const py::object &name_
 		bind_parameters["dateformat"] = Value(py::str(date_format));
 	}
 
+	if (!py::none().is(auto_detect)) {
+		bool auto_detect_as_int = py::isinstance<py::int_>(auto_detect);
+		bool auto_detect_as_bool = py::isinstance<py::bool_>(auto_detect);
+		bool auto_detect_value;
+		if (auto_detect_as_bool) {
+			auto_detect_value = py::bool_(auto_detect);
+		} else if (auto_detect_as_int) {
+			if ((int)py::int_(auto_detect) != 0) {
+				throw InvalidInputException("read_csv only accepts 0 if 'auto_detect' is given as an integer");
+			}
+			auto_detect_value = true;
+		} else {
+			throw InvalidInputException("read_csv only accepts 'auto_detect' as an integer, or a boolean");
+		}
+		bind_parameters["auto_detect"] = Value::BOOLEAN(auto_detect_value);
+	}
+
 	if (!py::none().is(timestamp_format)) {
 		if (!py::isinstance<py::str>(timestamp_format)) {
 			throw InvalidInputException("read_csv only accepts 'timestamp_format' as a string");
@@ -1467,8 +1526,10 @@ unique_ptr<DuckDBPyRelation> DuckDBPyConnection::RunQuery(const py::object &quer
 
 	if (!relation) {
 		// Could not create a relation, resort to direct execution
-		auto prep = PrepareQuery(std::move(last_statement));
-		auto res = ExecuteInternal(*prep, std::move(params));
+		unique_ptr<QueryResult> res;
+
+		res = PrepareAndExecuteInternal(std::move(last_statement), std::move(params));
+
 		if (!res) {
 			return nullptr;
 		}
@@ -1502,16 +1563,74 @@ unique_ptr<DuckDBPyRelation> DuckDBPyConnection::Table(const string &tname) {
 	}
 }
 
-unique_ptr<DuckDBPyRelation> DuckDBPyConnection::Values(py::object params) {
+static vector<unique_ptr<ParsedExpression>> ValueListFromExpressions(const py::args &expressions) {
+	vector<unique_ptr<ParsedExpression>> result;
+	auto arg_count = expressions.size();
+	if (arg_count == 0) {
+		throw InvalidInputException("Please provide a non-empty tuple");
+	}
+
+	for (idx_t i = 0; i < arg_count; i++) {
+		py::handle arg = expressions[i];
+		shared_ptr<DuckDBPyExpression> py_expr;
+		if (!py::try_cast<shared_ptr<DuckDBPyExpression>>(arg, py_expr)) {
+			throw InvalidInputException("Please provide arguments of type Expression!");
+		}
+		auto expr = py_expr->GetExpression().Copy();
+		result.push_back(std::move(expr));
+	}
+	return result;
+}
+
+static vector<vector<unique_ptr<ParsedExpression>>> ValueListsFromTuples(const py::args &tuples) {
+	auto arg_count = tuples.size();
+	if (arg_count == 0) {
+		throw InvalidInputException("Please provide a non-empty tuple");
+	}
+
+	idx_t expected_length = 0;
+	vector<vector<unique_ptr<ParsedExpression>>> result;
+	for (idx_t i = 0; i < arg_count; i++) {
+		py::handle arg = tuples[i];
+		if (!py::isinstance<py::tuple>(arg)) {
+			string actual_type = py::str(arg.get_type());
+			throw InvalidInputException("Expected objects of type tuple, not %s", actual_type);
+		}
+		auto expressions = py::cast<py::args>(arg);
+		auto value_list = ValueListFromExpressions(expressions);
+		if (i && value_list.size() != expected_length) {
+			throw InvalidInputException("Mismatch between length of tuples in input, expected %d but found %d",
+			                            expected_length, value_list.size());
+		}
+		expected_length = value_list.size();
+		result.push_back(std::move(value_list));
+	}
+	return result;
+}
+
+unique_ptr<DuckDBPyRelation> DuckDBPyConnection::Values(const py::args &args) {
 	auto &connection = con.GetConnection();
-	if (params.is_none()) {
-		params = py::list();
+
+	auto arg_count = args.size();
+	if (arg_count == 0) {
+		throw InvalidInputException("Could not create a ValueRelation without any inputs");
 	}
-	if (!py::hasattr(params, "__len__")) {
-		throw InvalidInputException("Type of object passed to parameter 'values' must be iterable");
+
+	D_ASSERT(py::gil_check());
+	py::handle first_arg = args[0];
+	if (arg_count == 1 && py::isinstance<py::list>(first_arg)) {
+		vector<vector<Value>> values {DuckDBPyConnection::TransformPythonParamList(first_arg)};
+		return make_uniq<DuckDBPyRelation>(connection.Values(values));
+	} else {
+		vector<vector<unique_ptr<ParsedExpression>>> expressions;
+		if (py::isinstance<py::tuple>(first_arg)) {
+			expressions = ValueListsFromTuples(args);
+		} else {
+			auto values = ValueListFromExpressions(args);
+			expressions.push_back(std::move(values));
+		}
+		return make_uniq<DuckDBPyRelation>(connection.Values(std::move(expressions)));
 	}
-	vector<vector<Value>> values {DuckDBPyConnection::TransformPythonParamList(params)};
-	return make_uniq<DuckDBPyRelation>(connection.Values(values));
 }
 
 unique_ptr<DuckDBPyRelation> DuckDBPyConnection::View(const string &vname) {
@@ -1611,40 +1730,6 @@ unique_ptr<DuckDBPyRelation> DuckDBPyConnection::FromArrow(py::object &arrow_obj
 	return make_uniq<DuckDBPyRelation>(std::move(rel));
 }
 
-unique_ptr<DuckDBPyRelation> DuckDBPyConnection::FromSubstrait(py::bytes &proto) {
-	auto &connection = con.GetConnection();
-	string name = "substrait_" + StringUtil::GenerateRandomName();
-	vector<Value> params;
-	params.emplace_back(Value::BLOB_RAW(proto));
-	return make_uniq<DuckDBPyRelation>(connection.TableFunction("from_substrait", params)->Alias(name));
-}
-
-unique_ptr<DuckDBPyRelation> DuckDBPyConnection::GetSubstrait(const string &query, bool enable_optimizer) {
-	auto &connection = con.GetConnection();
-	vector<Value> params;
-	params.emplace_back(query);
-	named_parameter_map_t named_parameters({{"enable_optimizer", Value::BOOLEAN(enable_optimizer)}});
-	return make_uniq<DuckDBPyRelation>(
-	    connection.TableFunction("get_substrait", params, named_parameters)->Alias(query));
-}
-
-unique_ptr<DuckDBPyRelation> DuckDBPyConnection::GetSubstraitJSON(const string &query, bool enable_optimizer) {
-	auto &connection = con.GetConnection();
-	vector<Value> params;
-	params.emplace_back(query);
-	named_parameter_map_t named_parameters({{"enable_optimizer", Value::BOOLEAN(enable_optimizer)}});
-	return make_uniq<DuckDBPyRelation>(
-	    connection.TableFunction("get_substrait_json", params, named_parameters)->Alias(query));
-}
-
-unique_ptr<DuckDBPyRelation> DuckDBPyConnection::FromSubstraitJSON(const string &json) {
-	auto &connection = con.GetConnection();
-	string name = "from_substrait_" + StringUtil::GenerateRandomName();
-	vector<Value> params;
-	params.emplace_back(json);
-	return make_uniq<DuckDBPyRelation>(connection.TableFunction("from_substrait_json", params)->Alias(name));
-}
-
 unordered_set<string> DuckDBPyConnection::GetTableNames(const string &query) {
 	auto &connection = con.GetConnection();
 	return connection.GetTableNames(query);
@@ -1765,6 +1850,20 @@ void DuckDBPyConnection::LoadExtension(const string &extension) {
 	ExtensionHelper::LoadExternalExtension(*connection.context, extension);
 }
 
+shared_ptr<DuckDBPyConnection> DefaultConnectionHolder::Get() {
+	lock_guard<mutex> guard(l);
+	if (!connection) {
+		py::dict config_dict;
+		connection = DuckDBPyConnection::Connect(py::str(":memory:"), false, config_dict);
+	}
+	return connection;
+}
+
+void DefaultConnectionHolder::Set(shared_ptr<DuckDBPyConnection> conn) {
+	lock_guard<mutex> guard(l);
+	connection = conn;
+}
+
 void DuckDBPyConnection::Cursors::AddCursor(shared_ptr<DuckDBPyConnection> conn) {
 	lock_guard<mutex> l(lock);
 
@@ -1799,6 +1898,8 @@ void DuckDBPyConnection::Cursors::ClearCursors() {
 		// release it don't ask me why it can't just realize there is no GIL and move on
 		py::gil_scoped_acquire gil;
 		cursor->Close();
+		// Ensure destructor runs with gil if triggered.
+		cursor.reset();
 	}
 
 	cursors.clear();
@@ -1886,8 +1987,11 @@ py::dict DuckDBPyConnection::FetchTF() {
 }
 
 PolarsDataFrame DuckDBPyConnection::FetchPolars(idx_t rows_per_batch) {
-	auto arrow = FetchArrow(rows_per_batch);
-	return py::cast<PolarsDataFrame>(py::module::import("polars").attr("DataFrame")(arrow));
+	if (!con.HasResult()) {
+		throw InvalidInputException("No open result set");
+	}
+	auto &result = con.GetResult();
+	return result.ToPolars(rows_per_batch);
 }
 
 duckdb::pyarrow::RecordBatchReader DuckDBPyConnection::FetchRecordBatchReader(const idx_t rows_per_batch) {
@@ -2041,11 +2145,11 @@ case_insensitive_map_t<BoundParameterData> DuckDBPyConnection::TransformPythonPa
 }
 
 shared_ptr<DuckDBPyConnection> DuckDBPyConnection::DefaultConnection() {
-	if (!default_connection) {
-		py::dict config_dict;
-		default_connection = DuckDBPyConnection::Connect(py::str(":memory:"), false, config_dict);
-	}
-	return default_connection;
+	return default_connection.Get();
+}
+
+void DuckDBPyConnection::SetDefaultConnection(shared_ptr<DuckDBPyConnection> connection) {
+	return default_connection.Set(std::move(connection));
 }
 
 PythonImportCache *DuckDBPyConnection::ImportCache() {
@@ -2090,7 +2194,7 @@ void DuckDBPyConnection::Exit(DuckDBPyConnection &self, const py::object &exc_ty
 }
 
 void DuckDBPyConnection::Cleanup() {
-	default_connection.reset();
+	default_connection.Set(nullptr);
 	import_cache.reset();
 }
 
@@ -2102,15 +2206,6 @@ bool DuckDBPyConnection::IsPandasDataframe(const py::object &object) {
 	return py::isinstance(object, import_cache_py.pandas.DataFrame());
 }
 
-bool DuckDBPyConnection::IsPolarsDataframe(const py::object &object) {
-	if (!ModuleIsLoaded<PolarsCacheItem>()) {
-		return false;
-	}
-	auto &import_cache_py = *DuckDBPyConnection::ImportCache();
-	return py::isinstance(object, import_cache_py.polars.DataFrame()) ||
-	       py::isinstance(object, import_cache_py.polars.LazyFrame());
-}
-
 bool IsValidNumpyDimensions(const py::handle &object, int &dim) {
 	// check the dimensions of numpy arrays
 	// should only be called by IsAcceptedNumpyObject
