@@ -2,7 +2,8 @@ import pytest
 
 _ = pytest.importorskip("duckdb.experimental.spark")
 
-from duckdb.experimental.spark.sql.types import Row
+from spark_namespace.sql.types import Row
+from spark_namespace import USE_ACTUAL_SPARK
 import textwrap
 
 
@@ -22,4 +23,9 @@ class TestSparkReadCSV(object):
         file_path = file_path.as_posix()
         df = spark.read.csv(file_path)
         res = df.collect()
-        assert res == [Row(column0=1, column1=2), Row(column0=3, column1=4), Row(column0=5, column1=6)]
+
+        expected_res = sorted([Row(column0=1, column1=2), Row(column0=3, column1=4), Row(column0=5, column1=6)])
+        if USE_ACTUAL_SPARK:
+            # Convert all values to strings as this is how Spark reads them by default
+            expected_res = [Row(column0=str(row.column0), column1=str(row.column1)) for row in expected_res]
+        assert sorted(res) == expected_res
